Package: shellgame
Title: The Shell Game - Audit Geographic Data Transformations
Version: 0.1.1
Authors@R: 
    person("Phinn", "Markson", , "markson.2@osu.edu", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-9169-6095"))
Description: Reveals how data quality silently degrades during geographic
    transformations while variable labels remain unchanged. Demonstrates
    that transformation error is agnostic to both the variable
    (population, income, etc.) and the tool ('R', 'Python', etc.). Provides a
    reproducible audit framework for quantifying the shift from observed
    to imputed data at each transformation hop.
License: MIT + file LICENSE
URL: https://github.com/phinnphace/shellgame
BugReports: https://github.com/phinnphace/shellgame/issues
Depends: R (>= 4.0.0)
Imports: dplyr (>= 1.0.0), ggplot2, janitor, magrittr, rlang, stringr,
        tidycensus, utils
Suggests: geoDeltaAudit, knitr, readr, rmarkdown, spelling, testthat
        (>= 3.0.0)
VignetteBuilder: knitr
Encoding: UTF-8
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
Language: en-US
NeedsCompilation: no
Packaged: 2026-05-20 16:58:57 UTC; phinnmarkson
Author: Phinn Markson [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-9169-6095>)
Maintainer: Phinn Markson <markson.2@osu.edu>
Repository: CRAN
Date/Publication: 2026-05-27 20:10:23 UTC
Built: R 4.5.2; ; 2026-05-27 20:54:53 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(print,shellgame_audit)
S3method(summary,shellgame_audit)
export(audit_transformation)
export(check_census_key)
export(create_audit_report)
export(extract_perturbed_population)
export(get_zcta_baseline)
export(pad_geoid)
export(plot_baseline_zctas)
export(plot_geometric_vs_relationship)
export(plot_transformation_perturbation)
export(prep_hud_crosswalk)
export(prep_zip_zcta)
export(run_full_transformation)
export(transform_zcta_to_zip)
export(transform_zip_to_county)
importFrom(magrittr,"%>%")

# shellgame <img src="man/figures/logo.png" align="right" height="139" />

[![CRAN status](https://www.r-pkg.org/badges/version/shellgame)](https://CRAN.R-project.org/package=shellgame)
[![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.18634426.svg)](https://doi.org/10.5281/zenodo.18634426)

> The column name stays the same. What it holds does not.

## Overview

Current practice in geographic data transformation suffers from three interrelated deficiencies. First, transformation error is rarely quantified. Researchers report that they "used HUD crosswalk Q3 2024" without acknowledging that this choice—among crosswalk source, time period, and allocation ratio—constitutes a methodological decision affecting results. Second, error is treated as incidental rather than structural. The prevailing assumption holds that crosswalks are neutral utilities, when in fact they encode assumptions about how population distributes across space that may not hold for the variable being studied. Third, equity implications remain unexamined. The differential impact of transformation choices on boundary communities and small populations is rarely discussed in published research.

The `shellgame` framework addresses these deficiencies. It provides a reproducible audit pipeline that quantifies transformation-induced error at each geographic "hop" and documents the hidden methodological decisions that directly affect research outcomes. The conceptual framework, implemented through the [`geoDeltaAudit`](https://CRAN.R-project.org/package=geoDeltaAudit) R package, enables researchers to transparently report transformation-induced uncertainty in population health studies. By making visible the assumptions embedded in standard transformation workflows, the framework supports more rigorous and equitable research practices.

## The Framework

The `shellgame` framework is built on the premise that geographic transformation is not a neutral operation but a sequence of decisions that alter the underlying data-generating process. The framework identifies two critical decision points that are typically left implicit in applied work.

**Decision 1: Membership Definition.** Researchers must choose whether to define geographic membership by administrative linkage (relationship-based, as used in ACS tabulations) or by geometric contact (spatial intersection).

**Decision 2: Crosswalk Selection.** When allocating ZIP-level data to counties, researchers must select among allocation ratios. The HUD ZIP-County crosswalk provides TOT_RATIO (total addresses), RES_RATIO (residential addresses only), BUS_RATIO (business addresses), and OTH_RATIO (other addresses). Each ratio embodies different assumptions about how population distributes across addresses that may not hold uniformly across urban, suburban, and rural contexts.

The framework quantifies delta_x(VAR), defined as the change in the value of a variable induced solely by geographic transformation and allocation choices, holding the underlying data source constant:

```
delta_x(VAR) = VAR₁ − VAR₀
```

Where VAR₀ is the baseline value at geography A, and VAR₁ is the post-transformation value. delta_x(VAR) does not imply directionality of truth — which representation is "correct" — nor that zero delta is inherently desirable. It states only that the variable is not invariant under this transformation.

Much like diagnostic formulation, where sensitivity and specificity must be balanced, geographic transformations operate within a similar trade-off space. A method that maximizes coverage may reduce geographic precision, while a method that preserves tight geographic alignment may shed or distort counts. Crosswalks, allocation rules, and lookup tables implicitly select a point along this trade-off. The audit makes that selection visible.

## The Hennepin County Case Study

Using 2022 ACS 5-year estimates for Hennepin County, Minnesota (n=74 ZCTAs, baseline population 1,391,557):

- **Pre-allocation expansion**: The ZCTA → ZIP hop alone increased the number of spatial units by 32.4% (74 → 98), before any weighting or allocation occurred. The analytical surface had already shifted.
- **Total transformation perturbation**: Following ZIP → County aggregation via HUD TOT_RATIO, the recovered population was 1,216,874 — a perturbation of **174,683 individuals (-12.6%)**.
- **Geographic redistribution**: The 174,683 not recovered in Hennepin County were allocated to five neighboring counties through crosswalk ratio weighting — made visible by the audit framework.

This perturbation is not an anomaly. It is a systematic consequence of transformation choices that researchers routinely make without quantifying.

## Installation

```r
install.packages("shellgame")
```

## Quick Start

```r
library(shellgame)

# Run audit on Hennepin County (pre-loaded example data)
result <- audit_transformation(
  baseline_data  = hennepin_zcta_baseline,
  zip_zcta_map   = hennepin_zip_zcta_map,
  hud_crosswalk  = hennepin_hud_crosswalk,
  county_fips    = "27053",
  variable_name  = "population"
)

summary(result)
#> === The Shell Game: Transformation Audit ===
#>
#> Variable: population
#> Target County: 27053
#>
#> --- Baseline (Observed Data) ---
#>   Units: 74 ZCTAs
#>   Total: 1,391,557
#>
#> --- After Transformation (Imputed Data) ---
#>   Intermediate: 98 ZIPs
#>   Recovered: 1,216,874
#>
#> --- The Shell Game Result ---
#>   Perturbation: -174,683 (-12.6%)
#>
#>   Same column name.
#>   Different underlying quantity.
#>   That's the shell game.

# Visualize
plot_transformation_perturbation(result)
```

## Equity Implications

The consequences of unquantified transformation perturbation are not distributed equally across communities. Communities located near administrative boundaries are systematically under- or over-represented depending on membership definitions. Small populations, including many racially and ethnically minoritized communities, are disproportionately affected by allocation assumptions that smooth heterogeneous distributions into uniform proxies. Historical undercounting in administrative data is amplified through successive transformations that treat imputed values as empirical measurements. When an analyst applies a crosswalk without auditing its effects, these distributional consequences remain invisible — buried in the methodological black box of geographic transformation.

## Related Package

[`geoDeltaAudit`](https://CRAN.R-project.org/package=geoDeltaAudit) is the operationalized implementation of the `shellgame` framework in R, providing the composable audit pipeline, step functions, and diagnostic output.

```r
install.packages(c("shellgame", "geoDeltaAudit"))
```

## Learn More

- `vignette("hennepin-example")` — Complete worked example
- `vignette("understanding-shell-game")` — Conceptual explanation
- `vignette("data-preparation", package = "geoDeltaAudit")` — How to audit your own geography

## Citation

```
Markson, P. (2026). shellgame: The Shell Game - Audit Geographic Data
  Transformations. R package version 0.1.1.
  https://CRAN.R-project.org/package=shellgame
```

## Inspiration

Package structure inspired by the excellent [`areal`](https://chris-prener.github.io/areal/) package by Christopher Prener.

## License

MIT © Phinn Markson

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)

## ----eval=FALSE---------------------------------------------------------------
# # Get your free key at: https://api.census.gov/data/key_signup.html
# # Then install it:
# tidycensus::census_api_key("YOUR_KEY_HERE", install = TRUE)

## ----eval=FALSE---------------------------------------------------------------
# library(zctaCrosswalk)
# 
# # Replace with your county FIPS code
# zctas <- get_zctas_by_county("27053") # Hennepin County, MN

## ----eval=FALSE---------------------------------------------------------------
# library(shellgame)
# 
# # Get total population (B01001_001)
# baseline_data <- get_zcta_baseline(
#     variable = "B01001_001",
#     year = 2022,
#     zctas = zctas
# )
# 
# # Or try other variables:
# # B19013_001 - Median household income
# # B25001_001 - Total housing units
# # B08201_001 - Households by vehicles available

## ----eval=FALSE---------------------------------------------------------------
# # Read the file
# zip_zcta_raw <- read.csv("ZiptoZCTA-Table 1.csv")
# 
# # Clean and standardize
# zip_zcta <- prep_zip_zcta(zip_zcta_raw)

## ----eval=FALSE---------------------------------------------------------------
# # Read the file
# hud_raw <- read.csv("ZIP_COUNTY_122024.csv")
# 
# # Clean and standardize
# hud <- prep_hud_crosswalk(hud_raw)
# 
# # Optional: use different ratio
# hud_res <- prep_hud_crosswalk(hud_raw, ratio_col = "RES_RATIO")

## ----eval=FALSE---------------------------------------------------------------
# result <- audit_transformation(
#     baseline_data = baseline_data,
#     zip_zcta_map = zip_zcta,
#     hud_crosswalk = hud,
#     county_fips = "27053",
#     variable_name = "population",
#     value_col = "estimate"
# )
# 
# summary(result)

## ----eval=FALSE---------------------------------------------------------------
# library(shellgame)
# library(zctaCrosswalk)
# library(tidycensus)
# 
# # Set your Census API key (one time)
# census_api_key("YOUR_KEY_HERE", install = TRUE)
# 
# # 1. Identify ZCTAs for your county
# zctas <- get_zctas_by_county("YOUR_COUNTY_FIPS")
# 
# # 2. Get baseline data
# baseline_data <- get_zcta_baseline(
#     variable = "B01001_001", # Total population
#     year = 2022,
#     zctas = zctas
# )
# 
# # 3. Prepare crosswalks
# zip_zcta <- prep_zip_zcta(read.csv("path/to/ZiptoZCTA.csv"))
# hud <- prep_hud_crosswalk(read.csv("path/to/ZIP_COUNTY.csv"))
# 
# # 4. Run audit
# result <- audit_transformation(
#     baseline_data = baseline_data,
#     zip_zcta_map = zip_zcta,
#     hud_crosswalk = hud,
#     county_fips = "YOUR_COUNTY_FIPS",
#     variable_name = "population"
# )
# 
# # 5. View results
# summary(result)
# plot_transformation_loss(result)

## ----eval=FALSE---------------------------------------------------------------
# # Example error:
# # Error: ZIP-ZCTA crosswalk is missing required columns: zcta

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  eval = FALSE,
  message = FALSE,
  warning = FALSE,
  error = FALSE
)

## ----libs---------------------------------------------------------------------
# library(shellgame)
# library(geoDeltaAudit)
# library(dplyr)
# library(stringr)
# library(janitor)
# 
# # vignette-only dependency; keep in Suggests
# if (!requireNamespace("readr", quietly = TRUE)) {
#   stop("Package 'readr' is required to run this vignette. Install it with install.packages('readr').")
# }

## ----eval=FALSE---------------------------------------------------------------
# acs_path  <- system.file("extdata", "toy_acs_zcta_hennepin.csv", package = "geoDeltaAudit")
# hud_path  <- system.file("extdata", "toy_zip_county_hud_hennepin.csv", package = "geoDeltaAudit")
# 
# stopifnot(nchar(acs_path) > 0, nchar(hud_path) > 0)
# 
# acs <- readr::read_csv(acs_path, show_col_types = FALSE) |>
#   janitor::clean_names() |>
#   dplyr::mutate(zcta = stringr::str_pad(as.character(.data$zcta), 5, pad = "0"))
# 
# hud <- readr::read_csv(hud_path, show_col_types = FALSE) |>
#   janitor::clean_names()
# 
# # Toy assoc: 1:1 ZCTA -> ZIP so the example always runs
# assoc <- acs |>
#   dplyr::distinct(.data$zcta) |>
#   dplyr::transmute(zcta = .data$zcta, zip = .data$zcta) |>
#   dplyr::distinct()
# 
# list(
#   acs_rows = nrow(acs),
#   assoc_rows = nrow(assoc),
#   hud_rows = nrow(hud)
# )

## ----run-audit, eval=FALSE, echo=TRUE-----------------------------------------
# # example only (not executed during vignette build)
# steps <- list(
#   geoDeltaAudit::step_zcta_to_zip_equal(assoc),
#   geoDeltaAudit::step_zip_to_county_totratio(hud = hud)
# )
# 
# out <- geoDeltaAudit::audit_transform(
#   df = acs,
#   geo_col = "zcta",
#   var_col = "pop",
#   steps = steps
# )

#' Audit geographic transformation
#'
#' Main function to audit a complete geographic transformation pipeline.
#' Quantifies the perturbation introduced at each hop and reveals the shell game.
#'
#' @param baseline_data Data frame with baseline data at source geography
#' @param zip_zcta_map ZIP-ZCTA association crosswalk
#' @param hud_crosswalk HUD ZIP-County crosswalk
#' @param county_fips Target county FIPS code
#' @param variable_name Name of the variable being tracked (for reporting)
#' @param value_col Name of the value column in baseline_data
#' @return An object of class "shellgame_audit" with audit results
#' @export
#' @examples
#' baseline <- data.frame(zcta = c("00001", "00002"), estimate = c(1000, 2000))
#' zip_zcta <- data.frame(zcta = c("00001", "00002"), zip = c("00010", "00010"))
#' hud <- data.frame(zip = "00010", county = "99999", tot_ratio = 1)
#' result <- audit_transformation(
#'     baseline_data = baseline,
#'     zip_zcta_map = zip_zcta,
#'     hud_crosswalk = hud,
#'     county_fips = "99999",
#'     variable_name = "population"
#' )
#' summary(result)
audit_transformation <- function(baseline_data, zip_zcta_map, hud_crosswalk,
                                 county_fips, variable_name = "value",
                                 value_col = "estimate") {
    # Calculate baseline total
    baseline_total <- sum(baseline_data[[value_col]], na.rm = TRUE)
    baseline_n_units <- nrow(baseline_data)

    # Run transformation pipeline
    transformed <- run_full_transformation(
        baseline_data = baseline_data,
        zip_zcta_map = zip_zcta_map,
        hud_crosswalk = hud_crosswalk,
        value_col = value_col,
        county_fips = county_fips
    )

    # Calculate recovered total
    recovered_total <- sum(transformed$county_level$value, na.rm = TRUE)

    # Calculate perturbation
    absolute_perturbation <- recovered_total - baseline_total
    percent_perturbation <- 100 * absolute_perturbation / baseline_total

    # Count units at each stage
    n_zips <- nrow(transformed$zip_level)

    # Calculate pre-allocation expansion
    pre_alloc_expansion <- (n_zips - baseline_n_units) / baseline_n_units * 100

    # Identify how population was redistributed across boundaries 
    perturbed_pop <- transformed$county_level %>%
        dplyr::filter(county != county_fips) %>%
        dplyr::arrange(dplyr::desc(value))

    # Create result object
    result <- list(
        # Summary metrics
        baseline_total = baseline_total,
        baseline_n_units = baseline_n_units,
        recovered_total = recovered_total,
        absolute_perturbation = absolute_perturbation,
        percent_perturbation = percent_perturbation,

        # Transformation details
        n_zips = n_zips,
        pre_alloc_expansion = pre_alloc_expansion,

        # Data at each stage
        baseline_data = baseline_data,
        zip_level = transformed$zip_level,
        county_level = transformed$county_level,
        perturbation_by_county = perturbed_pop,

        # Metadata
        county_fips = county_fips,
        variable_name = variable_name,
        value_col = value_col,

        # Crosswalks used
        zip_zcta_map = zip_zcta_map,
        hud_crosswalk = hud_crosswalk
    )

    class(result) <- "shellgame_audit"
    result
}

#' Print method for shellgame_audit
#'
#' @param x A shellgame_audit object
#' @param ... Additional arguments (ignored)
#' @return Invisibly returns the input object. Called for side effects (console output).
#' @export
print.shellgame_audit <- function(x, ...) {
    cat("\n=== The Shell Game: Transformation Audit ===\n\n")
    cat("Variable:", x$variable_name, "\n")
    cat("Target County:", x$county_fips, "\n\n")

    cat("--- Baseline (Observed Data) ---\n")
    cat("  Units:", x$baseline_n_units, "ZCTAs\n")
    cat("  Total:", format(round(x$baseline_total), big.mark = ","), "\n\n")

    cat("--- After Transformation (Imputed Data) ---\n")
    cat("  Intermediate: ", x$n_zips, " ZIPs\n")
    cat("  Recovered:", format(round(x$recovered_total), big.mark = ","), "\n\n")

    cat("--- The Shell Game Result ---\n")
    cat(
        "  Perturbation:", format(round(x$absolute_perturbation), big.mark = ","),
        sprintf("(%.1f%%)\n", x$percent_perturbation)
    )
    cat("\n")
    cat("  Same column name.\n")
    cat("  Different underlying quantity.\n")
    cat("  That's the shell game.\n\n")

    invisible(x)
}

#' Summary method for shellgame_audit
#'
#' @param object A shellgame_audit object
#' @param ... Additional arguments (ignored)
#' @return Invisibly returns the input object. Called for side effects (console output).
#' @export
summary.shellgame_audit <- function(object, ...) {
    print(object)

    cat("--- Pre-Allocation Expansion ---\n")
    cat(sprintf(
        "%d ZCTAs -> %d ZIPs (+%.1f%%)\n",
        object$baseline_n_units,
        object$n_zips,
        object$pre_alloc_expansion
    ))
    cat("  This happens BEFORE any allocation or weighting.\n")
    cat("  The analytical surface has already shifted.\n\n")

    if (nrow(object$perturbation_by_county) > 0) {
        cat("--- Perturbation by Receiving County ---\n")
        top_perturbed <- utils::head(object$perturbation_by_county, 5)
        for (i in seq_len(nrow(top_perturbed))) {
            cat(sprintf(
                "  %s: %s\n",
                top_perturbed$county[i],
                format(round(top_perturbed$value[i]), big.mark = ",")
            ))
        }
    }

    invisible(object)
}

#' Extract perturbation by receiving county
#'
#' Returns a data frame of counties that received population redistributed
#' from the target county during the transformation, ordered by magnitude.
#'
#' @param audit_result A shellgame_audit object
#' @param top_n Number of top counties to return (default: 10)
#' @return Data frame with columns: county, value
#' @export
extract_perturbed_population <- function(audit_result, top_n = 10) {
    audit_result$perturbation_by_county %>%
        utils::head(top_n)
}

test_that("audit_transformation returns a shellgame_audit object", {
  acs_path <- system.file("extdata", "toy_acs_zcta_hennepin.csv", package = "geoDeltaAudit")
  hud_path <- system.file("extdata", "toy_zip_county_hud_hennepin.csv", package = "geoDeltaAudit")

  skip_if(nchar(acs_path) == 0, "geoDeltaAudit toy data not available")

  acs <- readr::read_csv(acs_path, show_col_types = FALSE) |>
    janitor::clean_names() |>
    dplyr::mutate(zcta = stringr::str_pad(as.character(zcta), 5, pad = "0"))

  hud <- readr::read_csv(hud_path, show_col_types = FALSE) |>
    janitor::clean_names() |>
    dplyr::mutate(zip = stringr::str_pad(as.character(zip), 5, pad = "0"))

  assoc <- acs |>
    dplyr::distinct(zcta) |>
    dplyr::transmute(zcta = zcta, zip = zcta)

  result <- audit_transformation(
    baseline_data  = acs,
    zip_zcta_map   = assoc,
    hud_crosswalk  = hud,
    county_fips    = "27053",
    variable_name  = "population",
    value_col      = "pop"
  )

  expect_s3_class(result, "shellgame_audit")
})

test_that("audit_transformation result contains expected fields", {
  acs_path <- system.file("extdata", "toy_acs_zcta_hennepin.csv", package = "geoDeltaAudit")
  hud_path <- system.file("extdata", "toy_zip_county_hud_hennepin.csv", package = "geoDeltaAudit")

  skip_if(nchar(acs_path) == 0, "geoDeltaAudit toy data not available")

  acs <- readr::read_csv(acs_path, show_col_types = FALSE) |>
    janitor::clean_names() |>
    dplyr::mutate(zcta = stringr::str_pad(as.character(zcta), 5, pad = "0"))

  hud <- readr::read_csv(hud_path, show_col_types = FALSE) |>
    janitor::clean_names() |>
    dplyr::mutate(zip = stringr::str_pad(as.character(zip), 5, pad = "0"))

  assoc <- acs |>
    dplyr::distinct(zcta) |>
    dplyr::transmute(zcta = zcta, zip = zcta)

  result <- audit_transformation(
    baseline_data  = acs,
    zip_zcta_map   = assoc,
    hud_crosswalk  = hud,
    county_fips    = "27053",
    variable_name  = "population",
    value_col      = "pop"
  )

  expect_true(is.numeric(result$baseline_total))
  expect_true(is.numeric(result$recovered_total))
  expect_true(is.numeric(result$percent_perturbation))
  expect_true(is.numeric(result$absolute_perturbation))
  expect_true(result$baseline_total > 0)
  expect_equal(result$county_fips, "27053")
  expect_equal(result$variable_name, "population")
})

test_that("print and summary run without error", {
  acs_path <- system.file("extdata", "toy_acs_zcta_hennepin.csv", package = "geoDeltaAudit")
  hud_path <- system.file("extdata", "toy_zip_county_hud_hennepin.csv", package = "geoDeltaAudit")

  skip_if(nchar(acs_path) == 0, "geoDeltaAudit toy data not available")

  acs <- readr::read_csv(acs_path, show_col_types = FALSE) |>
    janitor::clean_names() |>
    dplyr::mutate(zcta = stringr::str_pad(as.character(zcta), 5, pad = "0"))

  hud <- readr::read_csv(hud_path, show_col_types = FALSE) |>
    janitor::clean_names() |>
    dplyr::mutate(zip = stringr::str_pad(as.character(zip), 5, pad = "0"))

  assoc <- acs |>
    dplyr::distinct(zcta) |>
    dplyr::transmute(zcta = zcta, zip = zcta)

  result <- audit_transformation(
    baseline_data  = acs,
    zip_zcta_map   = assoc,
    hud_crosswalk  = hud,
    county_fips    = "27053",
    variable_name  = "population",
    value_col      = "pop"
  )

  expect_no_error(print(result))
  expect_no_error(summary(result))
})

test_that("extract_perturbed_population returns a data frame respecting top_n", {
  acs_path <- system.file("extdata", "toy_acs_zcta_hennepin.csv", package = "geoDeltaAudit")
  hud_path <- system.file("extdata", "toy_zip_county_hud_hennepin.csv", package = "geoDeltaAudit")

  skip_if(nchar(acs_path) == 0, "geoDeltaAudit toy data not available")

  acs <- readr::read_csv(acs_path, show_col_types = FALSE) |>
    janitor::clean_names() |>
    dplyr::mutate(zcta = stringr::str_pad(as.character(zcta), 5, pad = "0"))

  hud <- readr::read_csv(hud_path, show_col_types = FALSE) |>
    janitor::clean_names() |>
    dplyr::mutate(zip = stringr::str_pad(as.character(zip), 5, pad = "0"))

  assoc <- acs |>
    dplyr::distinct(zcta) |>
    dplyr::transmute(zcta = zcta, zip = zcta)

  result <- audit_transformation(
    baseline_data  = acs,
    zip_zcta_map   = assoc,
    hud_crosswalk  = hud,
    county_fips    = "27053",
    variable_name  = "population",
    value_col      = "pop"
  )

  perturbed <- extract_perturbed_population(result, top_n = 3)

  expect_true(is.data.frame(perturbed))
  expect_lte(nrow(perturbed), 3)
})

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  eval = FALSE,
  message = FALSE,
  warning = FALSE,
  error = FALSE
)

## ----libs---------------------------------------------------------------------
# library(shellgame)
# library(geoDeltaAudit)
# library(dplyr)
# library(stringr)
# library(janitor)
# 
# # vignette-only dependency; keep in Suggests
# if (!requireNamespace("readr", quietly = TRUE)) {
#   stop("Package 'readr' is required to run this vignette. Install it with install.packages('readr').")
# }

## ----eval=FALSE---------------------------------------------------------------
# acs_path <- system.file("extdata", "toy_acs_zcta_hennepin.csv", package = "geoDeltaAudit")
# hud_path <- system.file("extdata", "toy_zip_county_hud_hennepin.csv", package = "geoDeltaAudit")
# 
# stopifnot(nchar(acs_path) > 0, nchar(hud_path) > 0)
# 
# acs <- readr::read_csv(acs_path, show_col_types = FALSE) |>
#   janitor::clean_names() |>
#   dplyr::mutate(zcta = stringr::str_pad(as.character(.data$zcta), 5, pad = "0"))
# 
# hud <- readr::read_csv(hud_path, show_col_types = FALSE) |>
#   janitor::clean_names()
# 
# # Toy assoc: 1:1 ZCTA -> ZIP so the example always runs
# assoc <- acs |>
#   dplyr::distinct(.data$zcta) |>
#   dplyr::transmute(zcta = .data$zcta, zip = .data$zcta) |>
#   dplyr::distinct()
# 
# list(
#   acs_rows = nrow(acs),
#   assoc_rows = nrow(assoc),
#   hud_rows = nrow(hud)
# )

## ----run-audit, eval=FALSE, echo=TRUE-----------------------------------------
# # example only (not executed during vignette build)
# result <- shellgame::evaluate_transformation(
#   data = acs,
#   zip_zcta_map = assoc,
#   hud_crosswalk = hud,
#   geo_col = "zcta",
#   var_col = "pop"
# )

## ----eval=FALSE---------------------------------------------------------------
# # Print summary
# summary(result)

## ----figures, echo=FALSE, eval=TRUE-------------------------------------------
knitr::include_graphics(c(
  "baseline_hennepin.png",
  "hennepin_relationship.png"
))

## ----baseline-fig, echo=FALSE, eval=TRUE--------------------------------------
knitr::include_graphics("baseline_hennepin.png")

## ----eval=FALSE---------------------------------------------------------------
# extract_perturbed_population(result, top_n = 5)

## ----baseline-membership, echo=FALSE, eval=TRUE-------------------------------
knitr::include_graphics("baseline_hennepin.png")

## ----geometric-membership, echo=FALSE, eval=TRUE------------------------------
knitr::include_graphics("hennepin_relationship.png")

## ----eval=FALSE---------------------------------------------------------------
# # Normalize expected fields from geoDeltaAudit::audit_transform()
# baseline_total <- as.numeric(audit_result$baseline_total)
# final_total <- as.numeric(audit_result$final_total)
# 
# # delta is already provided; compute if missing
# delta <- if (!is.null(audit_result$delta)) {
#   as.numeric(audit_result$delta)
# } else {
#   final_total - baseline_total
# }
# 
# absolute_perturbation <- abs(delta)

#' Pad GEOID to 5 digits
#'
#' Ensures geographic identifiers are zero-padded to 5 digits.
#'
#' @param geoid Character or numeric vector of geographic identifiers
#' @return Character vector of 5-digit zero-padded GEOIDs
#' @export
#' @examples
#' pad_geoid(c("123", "45678", 789))
#' #> [1] "00123" "45678" "00789"
pad_geoid <- function(geoid) {
    stringr::str_pad(as.character(geoid), 5, pad = "0")
}

#' Check for Census API key
#'
#' Validates that a Census API key is available for tidycensus.
#'
#' @param install Logical, whether to install the key for future sessions
#' @return Invisible TRUE if key exists, stops with error if not
#' @export
check_census_key <- function(install = FALSE) {
    key <- Sys.getenv("CENSUS_API_KEY")

    if (key == "") {
        stop(
            "No Census API key found. ",
            "Get one at https://api.census.gov/data/key_signup.html\\n",
            "Then set it with: tidycensus::census_api_key('YOUR_KEY', install = TRUE)",
            call. = FALSE
        )
    }

    invisible(TRUE)
}

#' Validate required columns in data frame
#'
#' @param data Data frame to validate
#' @param required_cols Character vector of required column names
#' @param data_name Name of the data object (for error messages)
#' @return Invisible TRUE if valid, stops with error if not
#' @keywords internal
#' @noRd 
validate_columns <- function(data, required_cols, data_name = "data") {
    missing_cols <- setdiff(required_cols, names(data))

    if (length(missing_cols) > 0) {
        stop(
            data_name, " is missing required columns: ",
            paste(missing_cols, collapse = ", "),
            call. = FALSE
        )
    }

    invisible(TRUE)
}

# Global variables used in NSE contexts
utils::globalVariables(c("ZCTA5CE10", "stage", "value", "type"))
#' @importFrom magrittr %>%
NULL
#' Plot baseline ZCTAs
#'
#' Creates a map showing the baseline ZCTAs used in the analysis.
#'
#' @param zcta_sf SF object with ZCTA geometries
#' @param county_sf SF object with county boundary
#' @param title Plot title
#' @return A ggplot2 object
#' @export
plot_baseline_zctas <- function(zcta_sf, county_sf,
                                title = "Baseline ZCTAs") {
    ggplot2::ggplot() +
        ggplot2::geom_sf(data = zcta_sf, alpha = 0.35, fill = "lightblue") +
        ggplot2::geom_sf(data = county_sf, fill = NA, linewidth = 1) +
        ggplot2::theme_minimal() +
        ggplot2::labs(
            title = title,
            subtitle = sprintf("n = %d ZCTAs", nrow(zcta_sf))
        ) +
        ggplot2::theme(
            plot.title.position = "plot",
            plot.title = ggplot2::element_text(margin = ggplot2::margin(b = 5)),
            plot.subtitle = ggplot2::element_text(margin = ggplot2::margin(b = 10))
        )
}

#' Plot geometric vs relationship membership
#'
#' Visualizes the discrepancy between geometric intersection and
#' relationship-based membership.
#'
#' @param zcta_baseline_sf SF object with baseline ZCTAs (relationship-based)
#' @param zcta_geometric_sf SF object with all geometrically intersecting ZCTAs
#' @param county_sf SF object with county boundary
#' @param title Plot title
#' @return A ggplot2 object
#' @export
plot_geometric_vs_relationship <- function(zcta_baseline_sf, zcta_geometric_sf,
                                           county_sf,
                                           title = "Geometric vs Relationship Membership") {
    baseline_ids <- zcta_baseline_sf$ZCTA5CE10
    extra_zctas <- zcta_geometric_sf %>%
        dplyr::filter(!(ZCTA5CE10 %in% baseline_ids))
    ggplot2::ggplot() +
        ggplot2::geom_sf(data = zcta_baseline_sf, fill = NA, linewidth = 0.25) +
        ggplot2::geom_sf(data = extra_zctas, fill = "grey60", linewidth = 0.25) +
        ggplot2::geom_sf(data = county_sf, fill = NA, linewidth = 1.0) +
        ggplot2::theme_minimal() +
        ggplot2::labs(
            title = title,
            subtitle = sprintf(
                "Geometric: %d ZCTAs | Relationship: %d | Geometry-only: %d",
                nrow(zcta_geometric_sf),
                nrow(zcta_baseline_sf),
                nrow(extra_zctas)
            ),
            caption = "Grey polygons: appear only under geometric intersection"
        ) +
        ggplot2::theme(
            plot.title.position = "plot",
            plot.title = ggplot2::element_text(margin = ggplot2::margin(b = 5)),
            plot.subtitle = ggplot2::element_text(margin = ggplot2::margin(b = 10)),
            plot.caption = ggplot2::element_text(margin = ggplot2::margin(t = 10))
        )
}

#' Plot transformation perturbation
#'
#' Creates a simple bar chart showing baseline vs recovered values.
#'
#' @param audit_result A shellgame_audit object
#' @return A ggplot2 object
#' @export
plot_transformation_perturbation <- function(audit_result) {
    plot_data <- data.frame(
        stage = c("Baseline\n(Observed)", "After Transformation\n(Imputed)"),
        value = c(audit_result$baseline_total, audit_result$recovered_total),
        type = c("Observed", "Imputed")
    )
    ggplot2::ggplot(plot_data, ggplot2::aes(x = stage, y = value, fill = type)) +
        ggplot2::geom_col() +
        ggplot2::geom_text(
            ggplot2::aes(label = format(round(value), big.mark = ",")),
            vjust = -0.5
        ) +
        ggplot2::scale_fill_manual(
            values = c("Observed" = "#4CAF50", "Imputed" = "#FF9800")
        ) +
        ggplot2::theme_minimal() +
        ggplot2::labs(
            title = "The Shell Game",
            subtitle = sprintf(
                "%s: Perturbation of %s (%.1f%%)",
                audit_result$variable_name,
                format(round(abs(audit_result$absolute_perturbation)), big.mark = ","),
                abs(audit_result$percent_perturbation)
            ),
            x = NULL,
            y = audit_result$variable_name,
            fill = "Data Type",
            caption = "Same column name. Different underlying quantity."
        ) +
        ggplot2::theme(
            legend.position = "bottom",
            plot.title.position = "plot",
            plot.title = ggplot2::element_text(margin = ggplot2::margin(b = 5)),
            plot.subtitle = ggplot2::element_text(margin = ggplot2::margin(b = 10)),
            plot.caption = ggplot2::element_text(margin = ggplot2::margin(t = 10))
        )
}

#' Create complete audit report
#'
#' Generates all visualizations for an audit.
#'
#' @param audit_result A shellgame_audit object
#' @param zcta_baseline_sf Optional: SF object with baseline ZCTAs
#' @param zcta_geometric_sf Optional: SF object with geometric ZCTAs
#' @param county_sf Optional: SF object with county boundary
#' @return List of ggplot2 objects
#' @export
create_audit_report <- function(audit_result, zcta_baseline_sf = NULL,
                                zcta_geometric_sf = NULL, county_sf = NULL) {
    plots <- list()
    # Always create the perturbation plot
    plots$perturbation <- plot_transformation_perturbation(audit_result)
    # Create spatial plots if geometries provided
    if (!is.null(zcta_baseline_sf) && !is.null(county_sf)) {
        plots$baseline <- plot_baseline_zctas(zcta_baseline_sf, county_sf)
    }
    if (!is.null(zcta_baseline_sf) && !is.null(zcta_geometric_sf) && !is.null(county_sf)) {
        plots$membership <- plot_geometric_vs_relationship(
            zcta_baseline_sf, zcta_geometric_sf, county_sf
        )
    }
    plots
}

#' Prepare ZIP-ZCTA crosswalk data
#'
#' Standardizes ZIP-ZCTA crosswalk data with proper column names and formatting.
#'
#' @param data Raw ZIP-ZCTA crosswalk data frame
#' @param zip_col Name of the ZIP code column (default: "ZIP_CODE" or "zip")
#' @param zcta_col Name of the ZCTA column (default: "zcta")
#' @return Data frame with standardized columns: zcta, zip
#' @export
#' @examples
#' raw <- data.frame(ZIP_CODE = c("00010", "00010"), zcta = c("00001", "00002"))
#' result <- prep_zip_zcta(raw)
prep_zip_zcta <- function(data, zip_col = NULL, zcta_col = "zcta") {
    # Clean column names
    data <- janitor::clean_names(data)

    # Auto-detect ZIP column if not specified
    if (is.null(zip_col)) {
        if ("zip_code" %in% names(data)) {
            zip_col <- "zip_code"
        } else if ("zip" %in% names(data)) {
            zip_col <- "zip"
        } else {
            stop("Could not auto-detect ZIP column. Please specify zip_col.", call. = FALSE)
        }
    }

    # Validate required columns exist
    validate_columns(data, c(zip_col, zcta_col), "ZIP-ZCTA crosswalk")

    # Standardize and return
    result <- data %>%
        dplyr::transmute(
            zcta = pad_geoid(!!rlang::sym(zcta_col)),
            zip  = pad_geoid(!!rlang::sym(zip_col))
        ) %>%
        dplyr::distinct()

    # Remove rows with NA ZCTAs
    n_na <- sum(is.na(result$zcta))
    if (n_na > 0) {
        message("Removing ", n_na, " rows with missing ZCTA values")
        result <- result %>% dplyr::filter(!is.na(zcta))
    }

    result
}

#' Prepare HUD ZIP-County crosswalk data
#'
#' Standardizes HUD crosswalk data with proper column names and formatting.
#'
#' @param data Raw HUD crosswalk data frame
#' @param ratio_col Name of the ratio column to use (default: "TOT_RATIO")
#' @return Data frame with standardized columns: zip, county, tot_ratio
#' @export
#' @examples
#' raw <- data.frame(ZIP = "00010", COUNTY = "99999", TOT_RATIO = 1)
#' result <- prep_hud_crosswalk(raw)
prep_hud_crosswalk <- function(data, ratio_col = "TOT_RATIO") {
    # Clean column names
    data <- janitor::clean_names(data)

    # Convert ratio_col to lowercase for matching
    ratio_col_clean <- tolower(ratio_col)

    # Validate required columns
    required <- c("zip", "county", ratio_col_clean)
    validate_columns(data, required, "HUD crosswalk")

    # Standardize and return
    data %>%
        dplyr::transmute(
            zip = pad_geoid(zip),
            county = as.character(county),
            tot_ratio = as.numeric(!!rlang::sym(ratio_col_clean))
        )
}

#' Get ACS baseline data for ZCTAs
#'
#' Fetches ACS 5-year estimates for a specified variable at the ZCTA level using
#' the Census API via the tidycensus package. Requires a Census API key
#' (see \url{https://api.census.gov/data/key_signup.html}) and the tidycensus
#' package to be installed.
#'
#' @param variable ACS variable code (e.g., "B01001_001" for total population)
#' @param year ACS year (default: 2022)
#' @param zctas Optional character vector of ZCTAs to filter to
#' @return Data frame with columns: zcta, estimate, moe
#' @export
#' @examples
#' \dontrun{
#' # get_zcta_baseline() retrieves ACS data via the Census API.
#' # See vignette("data-preparation", package = "geoDeltaAudit") for a full walkthrough.
#' pop_data <- get_zcta_baseline("B01001_001", year = 2022)
#' }

get_zcta_baseline <- function(variable, year = 2022, zctas = NULL) {
  if (!requireNamespace("tidycensus", quietly = TRUE)) {
    stop(
      "Package 'tidycensus' is required for get_zcta_baseline(). ",
      "Install it with: install.packages('tidycensus')",
      call. = FALSE
    )
  }
  check_census_key()

    # Fetch data from Census API
    acs_raw <- tidycensus::get_acs(
        geography = "zcta",
        variables = variable,
        year = year,
        survey = "acs5",
        cache_table = TRUE
    )

    # Standardize
    result <- acs_raw %>%
        dplyr::transmute(
            zcta = pad_geoid(GEOID),
            estimate = as.numeric(estimate),
            moe = as.numeric(moe)
        )

    # Filter to specific ZCTAs if provided
    if (!is.null(zctas)) {
        zctas_padded <- pad_geoid(zctas)
        result <- result %>%
            dplyr::filter(zcta %in% zctas_padded)
    }

    result
}

#' Transform ZCTA data to ZIP level
#'
#' Performs the first hop: ZCTA → ZIP using association-based allocation.
#' This is where the first swap occurs: observed data → imputed data.
#'
#' @param baseline_data Data frame with columns: zcta, and a value column
#' @param zip_zcta_map Data frame with columns: zcta, zip
#' @param value_col Name of the value column in baseline_data (default: "estimate")
#' @return Data frame with columns: zip, value (allocated to ZIP level)
#' @export
#' @examples
#' baseline <- data.frame(zcta = c("00001", "00002"), estimate = c(1000, 2000))
#' zip_zcta <- data.frame(zcta = c("00001", "00002"), zip = c("00010", "00010"))
#' result <- transform_zcta_to_zip(baseline, zip_zcta, value_col = "estimate")
transform_zcta_to_zip <- function(baseline_data, zip_zcta_map, value_col = "estimate") {
    # Validate inputs
    validate_columns(baseline_data, c("zcta", value_col), "baseline_data")
    validate_columns(zip_zcta_map, c("zcta", "zip"), "zip_zcta_map")

    # Join baseline to ZIP-ZCTA map
    result <- baseline_data %>%
        dplyr::left_join(zip_zcta_map, by = "zcta") %>%
        # Count how many ZIPs each ZCTA maps to
        dplyr::group_by(zcta) %>%
        dplyr::mutate(n_zip_assoc = dplyr::n_distinct(zip)) %>%
        dplyr::ungroup() %>%
        # Allocate value equally across associated ZIPs
        dplyr::mutate(
            value_allocated = !!rlang::sym(value_col) / n_zip_assoc
        ) %>%
        # Sum to ZIP level (in case multiple ZCTAs map to same ZIP)
        dplyr::group_by(zip) %>%
        dplyr::summarise(
            value = sum(value_allocated, na.rm = TRUE),
            .groups = "drop"
        )

    result
}

#' Transform ZIP data to County level
#'
#' Performs the second hop: ZIP → County using HUD TOT_RATIO allocation.
#' This is where the second swap occurs: further imputation via proxy.
#'
#' @param zip_data Data frame with columns: zip, value
#' @param hud_crosswalk Data frame with columns: zip, county, tot_ratio
#' @param county_fips Optional FIPS code to filter to specific county
#' @return Data frame with columns: county, value (allocated to county level)
#' @export
#' @examples
#' zip_data <- data.frame(zip = "00010", value = 3000)
#' hud <- data.frame(zip = "00010", county = "99999", tot_ratio = 1)
#' result <- transform_zip_to_county(zip_data, hud, county_fips = "99999")
transform_zip_to_county <- function(zip_data, hud_crosswalk, county_fips = NULL) {
    # Validate inputs
    validate_columns(zip_data, c("zip", "value"), "zip_data")
    validate_columns(hud_crosswalk, c("zip", "county", "tot_ratio"), "hud_crosswalk")

    # Join ZIP data to HUD crosswalk
    result <- zip_data %>%
        dplyr::left_join(hud_crosswalk, by = "zip") %>%
        # Allocate value using TOT_RATIO
        dplyr::mutate(
            value_allocated = value * tot_ratio
        ) %>%
        # Sum to county level
        dplyr::group_by(county) %>%
        dplyr::summarise(
            value = sum(value_allocated, na.rm = TRUE),
            .groups = "drop"
        )

    # Filter to specific county if requested
    if (!is.null(county_fips)) {
        result <- result %>%
            dplyr::filter(county == county_fips)
    }

    result
}

#' Run full transformation pipeline
#'
#' Executes both hops: ZCTA → ZIP → County.
#' Tracks the complete swap from observed to imputed data.
#'
#' @param baseline_data Data frame with ZCTA-level baseline data
#' @param zip_zcta_map ZIP-ZCTA association table
#' @param hud_crosswalk HUD ZIP-County crosswalk
#' @param value_col Name of value column in baseline_data
#' @param county_fips Optional county FIPS to filter final result
#' @return List with intermediate and final results
#' @export
#' @examples
#' baseline <- data.frame(zcta = c("00001", "00002"), estimate = c(1000, 2000))
#' zip_zcta <- data.frame(zcta = c("00001", "00002"), zip = c("00010", "00010"))
#' hud <- data.frame(zip = "00010", county = "99999", tot_ratio = 1)
#' result <- run_full_transformation(baseline, zip_zcta, hud,
#'     value_col = "estimate", county_fips = "99999")
run_full_transformation <- function(baseline_data, zip_zcta_map, hud_crosswalk,
                                    value_col = "estimate", county_fips = NULL) {
    # Hop 1: ZCTA → ZIP
    zip_data <- transform_zcta_to_zip(
        baseline_data = baseline_data,
        zip_zcta_map = zip_zcta_map,
        value_col = value_col
    )

    # Hop 2: ZIP → County
    county_data <- transform_zip_to_county(
        zip_data = zip_data,
        hud_crosswalk = hud_crosswalk,
        county_fips = county_fips
    )

    list(
        zip_level = zip_data,
        county_level = county_data
    )
}

---
title: "Hennepin example"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Hennepin example}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  eval = FALSE,
  message = FALSE,
  warning = FALSE,
  error = FALSE
)
```

```{r libs}
library(shellgame)
library(geoDeltaAudit)
library(dplyr)
library(stringr)
library(janitor)

# vignette-only dependency; keep in Suggests
if (!requireNamespace("readr", quietly = TRUE)) {
  stop("Package 'readr' is required to run this vignette. Install it with install.packages('readr').")
}
```

# Introduction

This vignette demonstrates a complete transformation audit using Hennepin County, Minnesota as an example. We'll track total population through the transformation chain:

**ZCTA → ZIP → COUNTY**

And reveal the shell game: same column name ("population"), different underlying quantity (observed → imputed).

# The Workflow

## Step 1: Prepare the Data

For this example, we'll use the data you would typically prepare:

```{r eval=FALSE}
acs_path <- system.file("extdata", "toy_acs_zcta_hennepin.csv", package = "geoDeltaAudit")
hud_path <- system.file("extdata", "toy_zip_county_hud_hennepin.csv", package = "geoDeltaAudit")

stopifnot(nchar(acs_path) > 0, nchar(hud_path) > 0)

acs <- readr::read_csv(acs_path, show_col_types = FALSE) |>
  janitor::clean_names() |>
  dplyr::mutate(zcta = stringr::str_pad(as.character(.data$zcta), 5, pad = "0"))

hud <- readr::read_csv(hud_path, show_col_types = FALSE) |>
  janitor::clean_names()

# Toy assoc: 1:1 ZCTA -> ZIP so the example always runs
assoc <- acs |>
  dplyr::distinct(.data$zcta) |>
  dplyr::transmute(zcta = .data$zcta, zip = .data$zcta) |>
  dplyr::distinct()

list(
  acs_rows = nrow(acs),
  assoc_rows = nrow(assoc),
  hud_rows = nrow(hud)
)
```

## Step 2: Run the Audit

```{r run-audit, eval=FALSE, echo=TRUE}
# example only (not executed during vignette build)
result <- shellgame::evaluate_transformation(
  data = acs,
  zip_zcta_map = assoc,
  hud_crosswalk = hud,
  geo_col = "zcta",
  var_col = "pop"
)
```

## Step 3: View Results

```{r eval=FALSE}
# Print summary
summary(result)
```
## Membership Visualization

*Note: The following graphics are pre-rendered from the configured Hennepin County example dataset to illustrate the spatial relationships being audited.*

```{r figures, echo=FALSE, eval=TRUE}
knitr::include_graphics(c(
  "baseline_hennepin.png",
  "hennepin_relationship.png"
))
```

```text
=== The Shell Game: Transformation Audit ===


Variable: population 
Target County: 27053 

--- Baseline (Observed Data) ---
  Units: 74 ZCTAs
  Total: 1,391,557 

--- After Transformation (Imputed Data) ---
  Intermediate:  98  ZIPs
  Recovered: 1,216,874 

--- The Shell Game Result ---
  Perturbation: -174,683 (-12.6%)

  Same column name.
  Different underlying quantity.
  That's the shell game.

--- Pre-Allocation Expansion ---
  74 ZCTAs → 98 ZIPs (+32.4%)
  This happens BEFORE any allocation or weighting.
  The analytical surface has already shifted.

--- Top Counties Receiving Perturbed Population ---
  27003: 30,535
  27139: 25,268
  27123: 21,835
  27171: 14,391
  27059: 9,526
```
## Baseline: 74 ZCTAs

The analysis begins with 74 ZCTAs that have a relationship-based membership with Hennepin County. These are the ZCTAs used by the Census Bureau in ACS tabulations.

**Total population: 1,391,557** (directly observed from ACS)

```{r baseline-fig, echo=FALSE, eval=TRUE}
knitr::include_graphics("baseline_hennepin.png")
```
## The First Hop: ZCTA → ZIP

When we associate these 74 ZCTAs with ZIP codes:

- ZCTA 55401 → 8 ZIPs
- ZCTA 55402 → 6 ZIPs  
- Several others → 2-5 ZIPs

**Result: 74 ZCTAs become 98 ZIPs (+32.4%)**

This happens **before any allocation**. The analytical surface has already shifted.

## The Second Hop: ZIP → County

Using HUD's TOT_RATIO, we allocate ZIP-level population to counties.

**Result: Population recovered for Hennepin County: 1,216,874**

## The Perturbation

**174,683 people (-12.6%) disappeared in the transformation.**

Where did they go? To neighboring counties:

```{r eval=FALSE}
extract_perturbed_population(result, top_n = 5)
```

# Geometric vs Relationship Membership


If we used geometric intersection instead of relationship-based membership, we would have 94 ZCTAs, not 74.

**This is Decision #1**: How do we define membership?

The 20 extra ZCTAs (shown in grey) intersect the county boundary geometrically but are not included in the relationship-based membership used by ACS.
# Visualizing the Difference

```{r baseline-membership, echo=FALSE, eval=TRUE}
knitr::include_graphics("baseline_hennepin.png")
```

The baseline: 74 ZCTAs with relationship-based membership.
```{r geometric-membership, echo=FALSE, eval=TRUE}
knitr::include_graphics("hennepin_relationship.png")
```

The difference: Grey areas show ZCTAs that appear only under geometric intersection.


# The Shell Game Revealed

```{r eval=FALSE}
# Normalize expected fields from geoDeltaAudit::audit_transform()
baseline_total <- as.numeric(audit_result$baseline_total)
final_total <- as.numeric(audit_result$final_total)

# delta is already provided; compute if missing
delta <- if (!is.null(audit_result$delta)) {
  as.numeric(audit_result$delta)
} else {
  final_total - baseline_total
}

absolute_perturbation <- abs(delta)
```

Same column name: "population"  
Different underlying quantity: observed → imputed

**That's the shell game.**

# Why This Matters

This error is **agnostic** to:

- **Variable**: Try this with median income (B19013_001) - same % error
- **Tool**: Run this in Python or Stata - same % error
- **Geography**: Try another county - same pattern

** Transformation is the cause, not the tool or variable.**

# Next Steps

- Try this audit with your own geography
- Test with different ACS variables
- Compare different crosswalk versions
- Document your hidden decisions

See `vignette("data-preparation")` for how to prepare your own data.
See `vignette("conceptual_framework-shell-game")` for the conceptual explanation.

---
title: "The Shell Game: conceptual framework"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{The Shell Game: conceptual framework}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)
```

# The Shell Game

**Swapping the VAR without changing the name or the formula.**

Same column name.  
Different underlying quantity.

**That's the shell game.**

## What the Shell Actually Is

The shell **isn't** geography.  
The shell **is** the variable label.

If this were just geometric mismatch, we would have:

- rounding error
- boundary error
- aggregation perturbation

But because the estimate silently shifts, what we're actually doing is:

**Imputing population via a proxy and continuing to treat it as observed.**

This is how the distortion compounds.

## The Critical Shift

**We're no longer propagating data.**  
**We're propagating assumptions.**

Assumptions are sticky. They don't decay; they **accumulate**.

When you apply standard administrative crosswalks in succession, each step amplifies the prior steps assumptions, and allocation choices. And, adds its own. In this example case study, shellgame is using HUD and TOT_RATIO, which assumes and allocates, residential address distributions as a proxy for population. The resulting estimates retain the population label despite no longer representing directly observed population quantities. As a result, successive users inherit increasingly imputed values while treating them as empirical measurements.

## The Transformation Chain

I am going to quantify mismatch **at each hop**, nothing more, nothing less.

**ZCTA → ZIP → COUNTY**

### Pre-allocation

Some Census tabulation areas are being asked to stand in for **multiple postal service geographies**, with no guidance on how population should be divided.

**This is the exact moment analysts silently switch from "joining data" to "inventing rules."**

At this stage, we:

**have not:**
- applied TOT_RATIO
- averaged anything
- redistributed population

**have only:**
- expanded relationships

**Yet already:**
- The unit count has changed
- The geography has fragmented
- The analytical surface has shifted

Any downstream "fix" is compensating for **damage already done**, not refining a neutral process.

## First Claim

We can already say, truthfully and precisely:

*"Using a lookup-style ZCTA-ZIP association increases the number of spatial units representing a county by 32% (74 ZCTAs to 98 ZIPs), prior to any allocation or weighting."*

## The Two Hidden Decisions

### Decision 1: Membership Definition

**Are we defining membership by administrative linkage, or by geometric contact?**

- 74 ZCTAs (relationship-based membership used in ACS)
- vs 94 ZCTAs (geometric intersection with county boundary)
- 20 ZCTAs excluded before any transformation begins

This affects the baseline before any allocation occurs.

### Decision 2: Crosswalk Selection

When you write "Used HUD crosswalk 3rd quarter 2024" - what decision was made?

- Choice of crosswalk source (HUD vs Census vs commercial)
- Choice of time period (Q3 2024 vs Q4 2024 vs 2023)
- Choice of allocation method (TOT_RATIO vs RES_RATIO vs BUS_RATIO)

Each choice produces different results, but they're rarely acknowledged as methodological decisions.

## The Result

For Hennepin County, Minnesota:

**Input**: Population at ZCTA level  
**Transformation**: ZCTA → ZIP → County  
**Output**: Population at each level  
**Result**: delta(Population) = Baseline - Recovered

- **Baseline**: 1,391,557 (74 ZCTAs, directly observed from ACS)
- **After ZCTA→ZIP→County**: 1,216,874 (recovered via HUD TOT_RATIO)
- **Perturbation**: 174,683 people (-12.6%)

## Why This Is Agnostic

The shell game happens regardless of:

- **Which variable**: Population, median income, vehicle ownership - same % error
- **Which tool**: R, Python, Stata, ArcGIS - same % error
- **Which geography**: Hennepin County or anywhere else - same pattern

**The transformation is the cause, not the tool or variable.**

The shell game happens regardless of who's shuffling the shells.

## What This Package Does

`shellgame` provides tools to:

1. **Quantify** the error at each transformation hop
2. **Reveal** where the swap from observed to imputed data occurs
3. **Demonstrate** that the error is agnostic to inputs
4. **Document** the hidden decisions in the workflow

Use it to audit yourself. Use it to understand what's really happening when you transform geographic data.

**Same column name. Different underlying quantity. That's the shell game.**

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_prep.R
\name{prep_hud_crosswalk}
\alias{prep_hud_crosswalk}
\title{Prepare HUD ZIP-County crosswalk data}
\usage{
prep_hud_crosswalk(data, ratio_col = "TOT_RATIO")
}
\arguments{
\item{data}{Raw HUD crosswalk data frame}

\item{ratio_col}{Name of the ratio column to use (default: "TOT_RATIO")}
}
\value{
Data frame with standardized columns: zip, county, tot_ratio
}
\description{
Standardizes HUD crosswalk data with proper column names and formatting.
}
\examples{
raw <- data.frame(ZIP = "00010", COUNTY = "99999", TOT_RATIO = 1)
result <- prep_hud_crosswalk(raw)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transformation.R
\name{transform_zip_to_county}
\alias{transform_zip_to_county}
\title{Transform ZIP data to County level}
\usage{
transform_zip_to_county(zip_data, hud_crosswalk, county_fips = NULL)
}
\arguments{
\item{zip_data}{Data frame with columns: zip, value}

\item{hud_crosswalk}{Data frame with columns: zip, county, tot_ratio}

\item{county_fips}{Optional FIPS code to filter to specific county}
}
\value{
Data frame with columns: county, value (allocated to county level)
}
\description{
Performs the second hop: ZIP → County using HUD TOT_RATIO allocation.
This is where the second swap occurs: further imputation via proxy.
}
\examples{
zip_data <- data.frame(zip = "00010", value = 3000)
hud <- data.frame(zip = "00010", county = "99999", tot_ratio = 1)
result <- transform_zip_to_county(zip_data, hud, county_fips = "99999")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{pad_geoid}
\alias{pad_geoid}
\title{Pad GEOID to 5 digits}
\usage{
pad_geoid(geoid)
}
\arguments{
\item{geoid}{Character or numeric vector of geographic identifiers}
}
\value{
Character vector of 5-digit zero-padded GEOIDs
}
\description{
Ensures geographic identifiers are zero-padded to 5 digits.
}
\examples{
pad_geoid(c("123", "45678", 789))
#> [1] "00123" "45678" "00789"
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{plot_transformation_perturbation}
\alias{plot_transformation_perturbation}
\title{Plot transformation perturbation}
\usage{
plot_transformation_perturbation(audit_result)
}
\arguments{
\item{audit_result}{A shellgame_audit object}
}
\value{
A ggplot2 object
}
\description{
Creates a simple bar chart showing baseline vs recovered values.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/audit.R
\name{extract_perturbed_population}
\alias{extract_perturbed_population}
\title{Extract perturbation by receiving county}
\usage{
extract_perturbed_population(audit_result, top_n = 10)
}
\arguments{
\item{audit_result}{A shellgame_audit object}

\item{top_n}{Number of top counties to return (default: 10)}
}
\value{
Data frame with columns: county, value
}
\description{
Returns a data frame of counties that received population redistributed
from the target county during the transformation, ordered by magnitude.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transformation.R
\name{run_full_transformation}
\alias{run_full_transformation}
\title{Run full transformation pipeline}
\usage{
run_full_transformation(
  baseline_data,
  zip_zcta_map,
  hud_crosswalk,
  value_col = "estimate",
  county_fips = NULL
)
}
\arguments{
\item{baseline_data}{Data frame with ZCTA-level baseline data}

\item{zip_zcta_map}{ZIP-ZCTA association table}

\item{hud_crosswalk}{HUD ZIP-County crosswalk}

\item{value_col}{Name of value column in baseline_data}

\item{county_fips}{Optional county FIPS to filter final result}
}
\value{
List with intermediate and final results
}
\description{
Executes both hops: ZCTA → ZIP → County.
Tracks the complete swap from observed to imputed data.
}
\examples{
baseline <- data.frame(zcta = c("00001", "00002"), estimate = c(1000, 2000))
zip_zcta <- data.frame(zcta = c("00001", "00002"), zip = c("00010", "00010"))
hud <- data.frame(zip = "00010", county = "99999", tot_ratio = 1)
result <- run_full_transformation(baseline, zip_zcta, hud,
    value_col = "estimate", county_fips = "99999")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_prep.R
\name{get_zcta_baseline}
\alias{get_zcta_baseline}
\title{Get ACS baseline data for ZCTAs}
\usage{
get_zcta_baseline(variable, year = 2022, zctas = NULL)
}
\arguments{
\item{variable}{ACS variable code (e.g., "B01001_001" for total population)}

\item{year}{ACS year (default: 2022)}

\item{zctas}{Optional character vector of ZCTAs to filter to}
}
\value{
Data frame with columns: zcta, estimate, moe
}
\description{
Fetches ACS 5-year estimates for a specified variable at the ZCTA level using
the Census API via the tidycensus package. Requires a Census API key
(see \url{https://api.census.gov/data/key_signup.html}) and the tidycensus
package to be installed.
}
\examples{
\dontrun{
# get_zcta_baseline() retrieves ACS data via the Census API.
# See vignette("data-preparation", package = "geoDeltaAudit") for a full walkthrough.
pop_data <- get_zcta_baseline("B01001_001", year = 2022)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transformation.R
\name{transform_zcta_to_zip}
\alias{transform_zcta_to_zip}
\title{Transform ZCTA data to ZIP level}
\usage{
transform_zcta_to_zip(baseline_data, zip_zcta_map, value_col = "estimate")
}
\arguments{
\item{baseline_data}{Data frame with columns: zcta, and a value column}

\item{zip_zcta_map}{Data frame with columns: zcta, zip}

\item{value_col}{Name of the value column in baseline_data (default: "estimate")}
}
\value{
Data frame with columns: zip, value (allocated to ZIP level)
}
\description{
Performs the first hop: ZCTA → ZIP using association-based allocation.
This is where the first swap occurs: observed data → imputed data.
}
\examples{
baseline <- data.frame(zcta = c("00001", "00002"), estimate = c(1000, 2000))
zip_zcta <- data.frame(zcta = c("00001", "00002"), zip = c("00010", "00010"))
result <- transform_zcta_to_zip(baseline, zip_zcta, value_col = "estimate")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/audit.R
\name{summary.shellgame_audit}
\alias{summary.shellgame_audit}
\title{Summary method for shellgame_audit}
\usage{
\method{summary}{shellgame_audit}(object, ...)
}
\arguments{
\item{object}{A shellgame_audit object}

\item{...}{Additional arguments (ignored)}
}
\value{
Invisibly returns the input object. Called for side effects (console output).
}
\description{
Summary method for shellgame_audit
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_census_key}
\alias{check_census_key}
\title{Check for Census API key}
\usage{
check_census_key(install = FALSE)
}
\arguments{
\item{install}{Logical, whether to install the key for future sessions}
}
\value{
Invisible TRUE if key exists, stops with error if not
}
\description{
Validates that a Census API key is available for tidycensus.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{plot_geometric_vs_relationship}
\alias{plot_geometric_vs_relationship}
\title{Plot geometric vs relationship membership}
\usage{
plot_geometric_vs_relationship(
  zcta_baseline_sf,
  zcta_geometric_sf,
  county_sf,
  title = "Geometric vs Relationship Membership"
)
}
\arguments{
\item{zcta_baseline_sf}{SF object with baseline ZCTAs (relationship-based)}

\item{zcta_geometric_sf}{SF object with all geometrically intersecting ZCTAs}

\item{county_sf}{SF object with county boundary}

\item{title}{Plot title}
}
\value{
A ggplot2 object
}
\description{
Visualizes the discrepancy between geometric intersection and
relationship-based membership.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/audit.R
\name{audit_transformation}
\alias{audit_transformation}
\title{Audit geographic transformation}
\usage{
audit_transformation(
  baseline_data,
  zip_zcta_map,
  hud_crosswalk,
  county_fips,
  variable_name = "value",
  value_col = "estimate"
)
}
\arguments{
\item{baseline_data}{Data frame with baseline data at source geography}

\item{zip_zcta_map}{ZIP-ZCTA association crosswalk}

\item{hud_crosswalk}{HUD ZIP-County crosswalk}

\item{county_fips}{Target county FIPS code}

\item{variable_name}{Name of the variable being tracked (for reporting)}

\item{value_col}{Name of the value column in baseline_data}
}
\value{
An object of class "shellgame_audit" with audit results
}
\description{
Main function to audit a complete geographic transformation pipeline.
Quantifies the perturbation introduced at each hop and reveals the shell game.
}
\examples{
baseline <- data.frame(zcta = c("00001", "00002"), estimate = c(1000, 2000))
zip_zcta <- data.frame(zcta = c("00001", "00002"), zip = c("00010", "00010"))
hud <- data.frame(zip = "00010", county = "99999", tot_ratio = 1)
result <- audit_transformation(
    baseline_data = baseline,
    zip_zcta_map = zip_zcta,
    hud_crosswalk = hud,
    county_fips = "99999",
    variable_name = "population"
)
summary(result)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{create_audit_report}
\alias{create_audit_report}
\title{Create complete audit report}
\usage{
create_audit_report(
  audit_result,
  zcta_baseline_sf = NULL,
  zcta_geometric_sf = NULL,
  county_sf = NULL
)
}
\arguments{
\item{audit_result}{A shellgame_audit object}

\item{zcta_baseline_sf}{Optional: SF object with baseline ZCTAs}

\item{zcta_geometric_sf}{Optional: SF object with geometric ZCTAs}

\item{county_sf}{Optional: SF object with county boundary}
}
\value{
List of ggplot2 objects
}
\description{
Generates all visualizations for an audit.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_prep.R
\name{prep_zip_zcta}
\alias{prep_zip_zcta}
\title{Prepare ZIP-ZCTA crosswalk data}
\usage{
prep_zip_zcta(data, zip_col = NULL, zcta_col = "zcta")
}
\arguments{
\item{data}{Raw ZIP-ZCTA crosswalk data frame}

\item{zip_col}{Name of the ZIP code column (default: "ZIP_CODE" or "zip")}

\item{zcta_col}{Name of the ZCTA column (default: "zcta")}
}
\value{
Data frame with standardized columns: zcta, zip
}
\description{
Standardizes ZIP-ZCTA crosswalk data with proper column names and formatting.
}
\examples{
raw <- data.frame(ZIP_CODE = c("00010", "00010"), zcta = c("00001", "00002"))
result <- prep_zip_zcta(raw)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{plot_baseline_zctas}
\alias{plot_baseline_zctas}
\title{Plot baseline ZCTAs}
\usage{
plot_baseline_zctas(zcta_sf, county_sf, title = "Baseline ZCTAs")
}
\arguments{
\item{zcta_sf}{SF object with ZCTA geometries}

\item{county_sf}{SF object with county boundary}

\item{title}{Plot title}
}
\value{
A ggplot2 object
}
\description{
Creates a map showing the baseline ZCTAs used in the analysis.
}
